<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * This is a one-line short description of the file
 *
 * You can have a rather longer description of the file as well,
 * if you like, and it can span multiple lines.
 *
 * @package    mod
 * @subpackage stupla
 * @copyright  2012 Rüdiger Krauße
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once(dirname(dirname(dirname(__FILE__))).'/config.php');
require_once(dirname(__FILE__).'/lib.php');

$id = required_param('id', PARAM_INT);   // course

$course = $DB->get_record('course', array('id' => $id), '*', MUST_EXIST);

require_course_login($course);

add_to_log($course->id, 'stupla', 'view all', 'index.php?id='.$course->id, '');

$coursecontext = get_context_instance(CONTEXT_COURSE, $course->id);

// Moodle 1.4+ requires sesskey to be passed in forms
if (isset($USER->sesskey)) {
    $sesskey = '<input type="hidden" name="sesskey" value="'.$USER->sesskey.'" />';
} else {
    $sesskey = '';
}

$PAGE->set_url('/mod/stupla/index.php', array('id' => $id));
$PAGE->set_title(format_string($course->fullname));
$PAGE->set_heading(format_string($course->fullname));
$PAGE->set_context($coursecontext);
$PAGE->navbar->add(get_string('modulenameplural', 'stupla'));

echo $OUTPUT->header();

/// Get all required stringsstupla

$strstuplas = get_string("modulenameplural", "stupla");
$strstupla  = get_string("modulename", "stupla");

/// Get all the appropriate data

if (! $stuplas = get_all_instances_in_course("stupla", $course)) {
    notice("There are no stuplas", "../../course/view.php?id=$course->id");
    die;
}

/// Print the list of instances (your module will probably extend this)

$table = new html_table();

if ($course->format == 'weeks') {
    $table->head  = array(get_string('week'));
    $table->align = array('center');
} else if ($course->format == 'topics') {
    $table->head  = array(get_string('topic'));
    $table->align = array('center');
} else {
    $table->head  = array();
    $table->align = array();
}

$strupdate = get_string('update');

$useUpdateColumn = has_capability('moodle/course:manageactivities', $coursecontext);

if ( $useUpdateColumn ) {
    array_push($table->head, $strupdate);
    array_push($table->align, "center");
}

array_push($table->head,
    get_string("name"),
    get_string("users")." / ".get_string('sessions', 'stupla')." (".get_string('deleted_sessions', 'stupla').")",
    get_string("report")
);
array_push($table->align,
    "left", "left", "center"
);

foreach ($stuplas as $stupla) {
    if (!$stupla->visible) {
        //Show dimmed if the mod is hidden
        $link = "<a class=\"dimmed\" href=\"view.php?id=$stupla->coursemodule\">$stupla->name</a>";
    } else {
        //Show normal if the mod is visible
        $link = "<a href=\"view.php?id=$stupla->coursemodule\">$stupla->name</a>";
    }

    $data = array ();

    if ($course->format=="weeks" || $course->format=="topics") {
        array_push($data, $stupla->section);
    }

    if ( $useUpdateColumn ) {
        $updatebutton = ''
        .   '<form target="'.$CFG->framename.'" method="get" action="'.$CFG->wwwroot.'/course/mod.php">'
        .   '<input type="hidden" name="update" value="'.$stupla->coursemodule.'" />'
        .   $sesskey
        .   '<input type="submit" value="'.$strupdate.'" />'
        .   '</form>'
        ;
        array_push($data, $updatebutton);
    }

    array_push($data, $link);

    array_push($data,
      $DB->count_records_select("stupla_session", "stupla = $stupla->id", null, "COUNT(DISTINCT userid)")." / ".
      $DB->count_records_select("stupla_session", "stupla = $stupla->id AND (flags & ".STUPLA_SESSION_FLAGS_DELETED.") = 0", null, "COUNT(DISTINCT id)")." (".
      $DB->count_records_select("stupla_session", "stupla = $stupla->id AND (flags & ".STUPLA_SESSION_FLAGS_DELETED.") != 0", null, "COUNT(DISTINCT id)").")");

    array_push($data,
      "<a href=\"prot/prot.php?id=$stupla->coursemodule\">".get_string("protocol", "stupla")."</a>");

    $table->data[] = $data;
}

echo '<br /><span id="markHeight"></span>';

echo $OUTPUT->heading(get_string('modulenameplural', 'stupla'), 2);
echo html_writer::table($table);
?>
<script type="text/javascript">
//<![CDATA[
  function absoluteTop(obj)
  {
    var w = obj.offsetTop;
    while ( obj.offsetParent )
    {
      obj = obj.offsetParent;
      w += obj.offsetTop;
    }
    return w;
  }

  var headsize = absoluteTop(document.getElementById("markHeight"));
  for (var i = 0; i < document.links.length; i++)
    if ( document.links[i].href.search(/prot\/prot.php/) != -1 )
      document.links[i].href = document.links[i].href+"&headsize="+headsize;

//]]>
</script>
<?php

/// Finish the page
echo $OUTPUT->footer();

?>
