<?php  // $Id: lib.php,v 1.1 2007/11/23 23:31:26 krausze Exp $
/**
 * Library of functions and constants for module stupla
 *
 * @author
 * @version $Id: lib.php,v 1.1 2007/11/23 23:31:26 krausze Exp $
 * @package stupla
 **/

error_reporting(E_ALL);

define("STUPLA_NO",  "0");
define("STUPLA_YES", "1");

// stupla flags
define("STUPLA_ISUTF8", 1);
define("STUPLA_USESUBUSERS", 2);

define("STUPLA_SESSIONSRESUMABLE", 4);
define("STUPLA_FORCEONESESSION", 8);

define("STUPLA_SESS_MASK", 4+8);
define("STUPLA_SESS_ONLYONE", 4+8);
define("STUPLA_SESS_PRIMARYONE", 8);
define("STUPLA_SESS_PRIMARYMULTI", 4);
define("STUPLA_SESS_ONLYMULTI", 0);


define ("STUPLA_TEXTSOURCE_FILENAME", "0");
define ("STUPLA_TEXTSOURCE_FILEPATH", "1");
define ("STUPLA_TEXTSOURCE_DATAJS", "2");
define ("STUPLA_TEXTSOURCE_SPECIFIC", "3");

define("STUPLA_LOCATION_COURSEFILES", "0");
define("STUPLA_LOCATION_SITEFILES",   "1");

$STUPLA_LOCATION = array (
    STUPLA_LOCATION_COURSEFILES => get_string("coursefiles"),
    STUPLA_LOCATION_SITEFILES   => get_string("sitefiles"),
);

define("STUPLA_SESSION_FLAGS_DELETED", 1);

define("STUPLA_LOGIN", 199);
define("STUPLA_NOTE", 198);
define("STUPLA_MARK", 197);
define("STUPLA_SHEET", 196);
define("STUPLA_PLAN", 195);
define("STUPLA_GENERIC", 194); // "prot, "# ..." - Actions (Material, Protocol)
define("STUPLA_SYSTERROR", 180);

/**
 * Returns the information on whether the module supports a feature
 *
 * @see plugin_supports() in lib/moodlelib.php
 * @param string $feature FEATURE_xx constant for requested feature
 * @return mixed true if the feature is supported, null if unknown
 */
function stupla_supports($feature) {
    switch($feature) {
        case FEATURE_MOD_INTRO:         return true;
        default:                        return null;
    }
}

/**
 * Given an object containing all the necessary data,
 * (defined by the form in mod.html) this function
 * will create a new instance and return the id number
 * of the new instance.
 *
 * @param object $instance An object from the form in mod.html
 * @return int The id of the newly inserted stupla record
 **/
function stupla_add_instance(stdClass $stupla, mod_stupla_mod_form $mform = null) {
    global $DB;

    $stupla->timemodified = time();
    $stupla->flags = 0;

    stupla_set_form_values($stupla, $mform);
    stupla_pack_subuserdata($stupla);

    if ( stupla_isUTF8($stupla) )
        $stupla->flags |= STUPLA_ISUTF8;
    switch ($stupla->namesource) {
        case STUPLA_TEXTSOURCE_FILENAME:
            $stupla->name = basename($stupla->reference);
            break;
        case STUPLA_TEXTSOURCE_FILEPATH:
            $stupla->name = str_replace('/', ' ', $stupla->reference);
            break;
        case STUPLA_TEXTSOURCE_DATAJS:
            $stupla->name = '';
        default:
            if (empty($stupla->name)) {
                $datajs = stupla_load_data_js($stupla);
                if ( $datajs !== false ) {
                    if ( preg_match("/var Title = \"([^\"]*)\"/", $datajs, $m) ) {
                        $stupla->name = $m[1];
                    }
                }
            }
    }

    return $DB->insert_record('stupla', $stupla);
}

/**
 * Updates an instance of the stupla in the database
 *
 * Given an object containing all the necessary data,
 * (defined by the form in mod_form.php) this function
 * will update an existing instance with new data.
 *
 * @param object $stupla An object from the form in mod_form.php
 * @param mod_stupla_mod_form $mform
 * @return boolean Success/Fail
 */
function stupla_update_instance(stdClass $stupla, mod_stupla_mod_form $mform = null) {
    global $DB;

    $stupla->timemodified = time();
    $stupla->id = $stupla->instance;

/*TODO:    unset($stupla->reference);

    $context = get_context_instance(CONTEXT_MODULE, $stupla->coursemodule);
    if ($filename = $mform->get_new_filename('referencefilepicker')) {
        echo "[[", $filename, "]]";
        if ($file = $mform->save_stored_file('referencefilepicker', $context->id, 'stupla', 'source', 0, '/', $filename, true)) {
        echo "[[", $file->get_filepath(), $file->get_filename(), "]]";
        print_r($file);
            $stupla->reference = '/'.$file->get_filename();
        } else {
            $stupla->reference = '';
        }
    } else {
        $stupla->reference = '';
    }*/

    stupla_set_form_values($stupla, $mform);
    stupla_pack_subuserdata($stupla);

    return $DB->update_record('stupla', $stupla);
}

/**
 * Takes the values from the form and packs it into the etest-object
 **/
function stupla_set_form_values(&$stupla, mod_stupla_mod_form $mform = null) {
    // open sand close time
    if ( !isset($stupla->enabletimeopen) || $stupla->enabletimeopen == 0 ) {
        $stupla->timeopen = 0;
    } else {
        $stupla->timeopen = make_timestamp(
            $stupla->openyear, $stupla->openmonth, $stupla->openday,
            $stupla->openhour, $stupla->openminute, 0
        );
    }

    if ( !isset($stupla->enabletimeclose) || $stupla->enabletimeclose == 0 ) {
        $stupla->timeclose = 0;
    } else {
        $stupla->timeclose = make_timestamp(
            $stupla->closeyear, $stupla->closemonth, $stupla->closeday,
            $stupla->closehour, $stupla->closeminute, 0
        );
    }

    // sessions resumable
    $stupla->flags &= ~STUPLA_SESS_MASK;
    $stupla->flags |= $stupla->sessionsResumable;

    // Subuserdata
    if ( isset($stupla->useSubusers) )
        $stupla->flags |= STUPLA_USESUBUSERS;

    $stupla->sudata = $mform->get_subuserdata();
//    echo "[["; print_r($mform->exportValues(array('su_type_0'))); echo "]]";
//    for ($i = 0; isset($mform->{"su_type_".$i}); $i++) {
//        echo "((", $mform->{"su_type_".$i}, "))";
//        if ( !isset($mform->{"su_del_".$i}) ) {
//            $stupla->sudata[] = array(
//                "name" => $mform->{"su_name_".$i},
//                "type" => $mform->{"su_type_".$i},
//                "must" => isset($mform->{"su_must_".$i}),
//                "comment" => $mform->{"su_comment_".$i},
//                "data" => $mform->{"su_data_".$i});
//        }
//    }

    if ( $stupla->refInternExtern == 0 ) { // == intern, overwrite reference
        $stupla->reference = '';
        $context = get_context_instance(CONTEXT_MODULE, $stupla->coursemodule);
        echo "((save: ", $context->id, "))";
        $sourcefile = null;

        if ($stupla->referenceInt) {
            $options = array('subdirs' => 1, 'maxbytes' => 0, 'maxfiles' => -1);
            file_save_draft_area_files($stupla->referenceInt, $context->id, 'mod_stupla', 'sourcefile', 0, $options);

            $fs = get_file_storage();
            $files = $fs->get_area_files($context->id, 'mod_stupla', 'sourcefile');

            // do we need to remove the draft files ?
            // otherwise the "files" table seems to get full of "draft" records
            // $fs->delete_area_files($context->id, 'user', 'draft', $stupla->sourceitemid);

            foreach ($files as $hash => $file) {
                if ($file->get_sortorder()==1) {
                    $stupla->reference = $file->get_filepath().$file->get_filename();
                    $sourcefile = $file;
                    break;
                }
            }
            unset($fs, $files, $file, $hash, $options);

        }
        if (is_null($sourcefile) || $stupla->reference=='' ) {
            // sourcefile was missing or not a recognized type - shouldn't happen !!
        }

    }
}

/**
 * Given an ID of an instance of this module,
 * this function will permanently delete the instance
 * and any data that depends on it.
 *
 * @param int $id Id of the module instance
 * @return boolean Success/Failure
 **/
function stupla_delete_instance($id) {
    global $DB;

    if (! $stupla = $DB->get_record("stupla", array('id' => $id)) ) {
        return false;
    }

    $result = true;

    # Delete any dependent records here #

    if (! $DB->delete_records("stupla", "id", "$stupla->id")) { // TODO ther ar mor things to delete?
        $result = false;
    }

    return $result;
}

/**
 * Return a small object with summary information about what a
 * user has done with a given particular instance of this module
 * Used for user activity reports.
 * $return->time = the time they did it
 * $return->info = a short text description
 *
 * @return null
 * @todo Finish documenting this function
 **/
function stupla_user_outline($course, $user, $mod, $stupla) {
    return "not implementet yet"; // TODO rk
}

/**
 * Print a detailed representation of what a user has done with
 * a given particular instance of this module, for user activity reports.
 *
 * @return boolean
 * @todo Finish documenting this function
 **/
function stupla_user_complete($course, $user, $mod, $stupla) {
    return true;
}

/**
 * Given a course and a time, this module should find recent activity
 * that has occurred in stupla activities and print it out.
 * Return true if there was output, or false is there was none.
 *
 * @uses $CFG
 * @return boolean
 * @todo Finish documenting this function
 **/
function stupla_print_recent_activity($course, $isteacher, $timestart) {
    global $CFG;

    return false;  //  True if anything was printed, otherwise false
}

/**
 * Function to be run periodically according to the moodle cron
 * This function searches for things that need to be done, such
 * as sending out mail, toggling flags etc ...
 *
 * @uses $CFG
 * @return boolean
 * @todo Finish documenting this function
 **/
function stupla_cron () {
    global $CFG;

    return true;
}

/**
 * Must return an array of grades for a given instance of this module,
 * indexed by user.  It also returns a maximum allowed grade.
 *
 * Example:
 *    $return->grades = array of grades;
 *    $return->maxgrade = maximum allowed grade;
 *
 *    return $return;
 *
 * @param int $stuplaid ID of an instance of this module
 * @return mixed Null or object with an array of grades and with the maximum grade
 **/
function stupla_grades($stuplaid) {
   return NULL;
}

/**
 * Must return an array of user records (all data) who are participants
 * for a given instance of stupla. Must include every user involved
 * in the instance, independient of his role (student, teacher, admin...)
 * See other modules as example.
 *
 * @param int $stuplaid ID of an instance of this module
 * @return mixed boolean/array of students
 **/
function stupla_get_participants($stuplaid) {
    return false;
}

/**
 * This function returns if a scale is being used by one stupla
 * it it has support for grading and scales. Commented code should be
 * modified if necessary. See forum, glossary or journal modules
 * as reference.
 *
 * @param int $stuplaid ID of an instance of this module
 * @return mixed
 * @todo Finish documenting this function
 **/
function stupla_scale_used ($stuplaid,$scaleid) {
    $return = false;

    //$rec = get_record("stupla","id","$stuplaid","scale","-$scaleid");
    //
    //if (!empty($rec)  && !empty($scaleid)) {
    //    $return = true;
    //}

    return $return;
}

/**
 * Extends the global navigation tree by adding stupla nodes if there is a relevant content
 *
 * This can be called by an AJAX request so do not rely on $PAGE as it might not be set up properly.
 *
 * @param navigation_node $navref An object representing the navigation tree node of the stupla module instance
 * @param stdclass $course
 * @param stdclass $module
 * @param stdclass $cm
 */
function stupla_extend_navigation(navigation_node $stuplanode, stdclass $course, stdclass $module, stdclass $cm) {
    global $CFG, $DB;

    $stupla = $DB->get_record('stupla', array('id' => $cm->instance), '*', MUST_EXIST);

    if (has_capability('mod/stupla:reviewallprotocols', $cm->context) || has_capability('mod/stupla:reviewmyprotocols', $cm->context)) {
        $stuplanode->add(
            get_string('protocol', 'stupla'),
            new moodle_url('/mod/stupla/prot/prot.php', array('stupla' => $stupla->id)),
            navigation_node::TYPE_SETTING, null, null, new pix_icon('i/report', ''));
    }
}

function stupla_add_action($data) {
    global $stupla, $coure, $a, $sessionid, $DB;
    $b = explode(" ", $data);
    $c = explode("&", $b[0]);
    $d = explode(".", $c[0]);
    $action = new object();
    $action->stupla = $stupla->id;
    $action->session = $sessionid;
    $action->starttime = time();
//    endtime int(10) unsigned NOT NULL default '0',
    $action->media = $d[0];
    $action->topic = $d[1];
    $action->nr = $d[2];
    $action->timestamp = $c[1];
    $action->duration = $c[2];
  if ( isset($c[3]) )
        $action->result = $c[3];
  if ( isset($b[1]) )
        $action->data = substr($data, strlen($b[0])+1);

    $DB->insert_record('stupla_action', $action);
}

function stupla_add_extra_action($media, $data) {
    global $stupla, $coure, $a, $sessionid, $DB;
    $actiondata = new stdClass;
    $actiondata->stupla = $stupla->id;
    $actiondata->session = $sessionid;
    $actiondata->starttime = time();
    $actiondata->media = $media;
    $actiondata->topic = -99;
    $actiondata->nr = -99;
    $actiondata->data = $data;
    $DB->insert_record('stupla_action', $actiondata);
}

function stupla_load_data_js(&$stupla)
{
    global $CFG;
    $compPath = stupla_compdirreference($stupla);
    if ( stupla_is_extern($stupla) ) {
        $res = @file_get_contents($compPath.'/data.js');
        if ( $res === false )
            $res = @file_get_contents($compPath.'/data8.js', "r");
        return $res;
    }
    else {
        $file = stupla_get_file_int($stupla, 'data.js', $compPath);
        if ( $file === false )
            $file = stupla_get_file_int($stupla, 'data8.js', $compPath);
        return ($file !== false) ? $file->get_content() : false;
    }
}
function stupla_isUTF8($stupla) {
    $compPath = stupla_compdirreference($stupla);
    if ( stupla_is_extern($stupla) ) {
        if ( ($handle = fopen($compPath."/data.js", "r")) !== false ) {
            fclose($handle);
            return false;
        }
        if ( ($handle = fopen($compPath."/data8.js", "r")) !== false ) {
            fclose($handle);
            return true;
        }
    }
    else {
        $file = stupla_get_file_int($stupla, 'data.js', $compPath);
        if ( $file !== false )
            return false;
        $file = stupla_get_file_int($stupla, 'data8.js', $compPath);
        if ( $file !== false )
            return true;
    }
    return false;
}

// subuserdata

function stupla_expand_subuserdata(&$stupla) {
    $s = isset($stupla->subuserdata) && $stupla->subuserdata != '' ? $stupla->subuserdata :
        get_string('firstname', 'stupla')."\x02text\x02true\x02\x02\x01".
        get_string('lastname', 'stupla')."\x02text\x02true\x02\x02\x01".
        get_string('birthday', 'stupla')."\x02date\x02false\x02".
        get_string('dd.mm.yyyy', 'stupla')."\x02\x01";
    $a = explode("\x01", $s);
    $stupla->sudata = array();
    $c = 0;
    foreach ($a as $ai) {
        if ( $ai == "" )
            continue;
        $b = explode("\x02", $ai."\x02\x02");
        $stupla->sudata[$c++] = array(
            "name" => $b[0],
            "type" => $b[1],
            "must" => ($b[2] == "true"),
            "comment" => $b[3],
            "data" => $b[4]);
    }
}

function stupla_pack_subuserdata(&$stupla) {
    if ( !isset($stupla->sudata) )
        return;
    $a = array();
    foreach ($stupla->sudata as $id => $data) {
        $s = "";
        $s = $data["name"]."\x02".$data["type"]."\x02".($data["must"] ? "true" : "false")."\x02".$data["comment"]."\x02";
        if ( $data["type"] == "combo" )
            $s .= $data["data"];
        $a[] = $s;
    }
    $stupla->subuserdata = join("\x01", $a);
}

function stupla_pluginfile($course, $cm, $context, $filearea, $args, $forcedownload) {
    global $CFG, $DB;

    if ($context->contextlevel != CONTEXT_MODULE) {
        return false;
    }

    if (!$stupla = $DB->get_record('stupla', array('id'=>$cm->instance))) {
        return false;
    }

    require_course_login($course, true, $cm);

    array_shift($args); // ignore itemid - caching only
    $fullpath = "/$context->id/mod_stupla/$filearea/0/".implode('/', $args);

    $fs = get_file_storage();
    if (!$file = $fs->get_file_by_hash(sha1($fullpath)) or $file->is_directory()) {
        return false;
    }

    // finally send the file
    send_stored_file($file, 0, 0, $forcedownload); // download MUST be forced - security!
}

function stupla_get_cmid(&$stupla) {
	if ( isset($stupla->coursemodule) )
		return get_context_instance(CONTEXT_MODULE, $stupla->coursemodule)->id;
    return get_context_instance(CONTEXT_MODULE, get_coursemodule_from_instance('stupla', $stupla->id, $stupla->course, false, MUST_EXIST)->id)->id;
}

function stupla_get_file_int($stupla, $filename, $path = false) {
    if ( $path !== false ) {
        $dir = $path;
        $base = $filename;
    }
    else {
        $dir = dirname($filename);
        $base = basename($filename);
    }
    // check trailing '/' at dir
    if ( substr($dir, -1) != '/' && substr($dir, -1) != '\\' )
        $dir .= '/';
    $fs = get_file_storage();
    return $fs->get_file(
        stupla_get_cmid($stupla),
        'mod_stupla',
        'sourcefile',
        0,
        $dir,
        $base);
}

/**
 * Delivers the url-dir to the comp-folder. Used by prot. With NO TRAILING "/".
 */
function stupla_compdirreference(&$stupla) {
    $compdir = dirname($stupla->reference);
    if ( stupla_is_short_start($stupla) ) {
        if ( $stupla->localpath != '' ) {
            if ( preg_match('/[\\\\\/]([^\\\\\/]+)$/', $stupla->localpath, $b) )
                $compdir .= '/'.$b[1];
        }
        else { // try to read this ..._start.htm
            if ( !stupla_is_extern($stupla) ) {
                $file = stupla_get_file_int($stupla, $stupla->reference);
                $source = $file->get_content();
                if ( preg_match("/<frame src=\"([^\"]*?)\/Con_d\.htm/", $source, $b) )
                {
                    $compdir .= $b[1];
                }
            }
        }
    }
    return $compdir;
}

function stupla_is_extern(&$stupla) {
    return preg_match('/^http(s|):\/\//', is_object($stupla) ? $stupla->reference : $stupla);
}

function stupla_is_short_start(&$stupla) {
    if ( stupla_is_extern($stupla) )
        $filename= $stupla->reference;
    else
    {
        $file = stupla_get_file_int($stupla, $stupla->reference);
        $filename = $file->get_filename();
    }
    return strcasecmp(substr($filename, strlen($filename)-10), '_start.htm') == 0;
}

function stupla_www_prefix(&$stupla) {
    global $CFG;
    if ( stupla_is_extern($stupla) )
        return '';
    return $CFG->wwwroot.'/pluginfile.php/'.stupla_get_cmid($stupla).'/mod_stupla/sourcefile/0';
}

?>