<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * This page is the protocol page providing an overview of all ussers/subusers.
 *
 * @package    mod
 * @subpackage stupla
 * @copyright  2012 Rüdiger Krauße
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

header("Expires: 0");

include 'protlib.php';
include 'data_load.php';
include 'prot_util.php';

$PAGE->set_url('/mod/stupla/prot/prot_userlist.php', array('id' => $cm->id));
$PAGE->navbar->ignore_active();

$table = new html_table();

$table->name = get_string('user_list', 'stupla');
$table->head = array(
                get_string('user'),
                get_string('logins', 'stupla'),
                get_string('fb_reqs', 'stupla'),
                get_string('texts', 'stupla'),
                get_string('texts_time', 'stupla'),
                get_string('media', 'stupla'),
                get_string('exercises', 'stupla'),
                get_string('time_total', 'stupla'),
);
$table->align = array('left', 'right', 'right', 'right', 'right', 'right', 'right', 'right');
$table->data = array();
$table->ordertype = array('a','n','n','n','', 'n','','');

$canDelete = 0;
if ( has_capability('mod/stupla:deletemysessions', $coursecontext) )
    $canDelete = 1;
else if ( has_capability('mod/stupla:deleteallsessions', $coursecontext) )
    $canDelete = 2;

$deletesession = optional_param('deletesession', -1, PARAM_INT);

$undeletemode = $canDelete > 0 ? optional_param('undeletemode', 0, PARAM_INT) : -1;

$undeletesession = optional_param('undeletesession', -1, PARAM_INT);

$notify = '';

if ( $canDelete > 0 && $deletesession >= 0 ) {
    // delete physical
    /*delete_records('stupla_action', 'session', $deletesession);
    delete_records('stupla_ex', 'session', $deletesession);
    delete_records('stupla_sheet', 'session', $deletesession);
    delete_records('stupla_plan', 'session', $deletesession);
    $res = delete_records('stupla_session', 'id', $deletesession);*/
    // delete logical
    $sessiondata = $DB->get_record('stupla_session', array('id' => $deletesession));
    $sessiondata->flags |= STUPLA_SESSION_FLAGS_DELETED;
    $res = $DB->update_record('stupla_session', $sessiondata);

    //print_string('phys_deleted', 'stupla', $deletesession);
    $notify = get_string('log_deleted', 'stupla', $deletesession);
}

if ( $undeletesession >= 0 ) {
    $sessiondata = $DB->get_record('stupla_session', array('id' => $undeletesession));
    $sessiondata->flags &= ~STUPLA_SESSION_FLAGS_DELETED;
    $res = $DB->update_record('stupla_session', $sessiondata);

    $notify = get_string('log_undeleted', 'stupla', $undeletesession);
}

// subuser data
if ( $stupla->flags & STUPLA_USESUBUSERS ) {
    stupla_expand_subuserdata($stupla);
    foreach($stupla->sudata as $field) {
        array_push($table->head, $field['name']);
        array_push($table->align, 'left');
        array_push($table->ordertype, '');
    }
}

// delete column
if ( $canDelete > 0 && $format == "showashtml" ) {
    array_push($table->head, '');
    array_push($table->align, 'left');
    array_push($table->ordertype, '');
}

$sError = "";

if ( $undeletemode == 1 )
    $activeClause = ' AND l.flags = 1';

$sessions = GetSessions($stupla);

foreach ($sessions as $session)
{
    ClearHist();
    ReadHist($session, $stupla);
    $d = array($format == "showashtml"
        ? '<a href="javascript:top.control.SelectHist(\''.$session->id.'\')" target="control">'.$session->displayname.'</a>'
        : $session->displayname);
    $d = array_merge($d, SummaryHist($format == "showashtml"));
    // subuserdata
    if ( $stupla->flags & STUPLA_USESUBUSERS ) {
        $subdat = isset($session->data) ? explode("\x01", $session->data) : array();
        for ($i = 0; $i < count($stupla->sudata); $i++)
            array_push($d, isset($subdat[$i]) ? $subdat[$i] : '');
    }
    if ( $format == "showashtml" ) {
        if ( $undeletemode == 1 )
            array_push($d, '<input type="button" onclick="undelete_session('.$session->id.')" value="'.get_string('undelete', 'stupla').'" />');
        else if ( $canDelete > 0 ) {
            if ( ($canDelete == 2) || ($USER->id == $session->userid) )
                array_push($d, '<input type="button" onclick="delete_session('.$session->id.', \''.$session->displayname.'\')" value="'.get_string('delete', 'stupla').'" />');
            else
                array_push($d, '');
        }
    }

    // append line to table
    $table->data[] = $d;
}

$tables = array($table);

if ( $format == "showashtml" ) {
    echo $OUTPUT->header();
    if ( $notify != '' )
        echo '<p class="notifysuccess">', $notify, '</p>';
    ?>
    <script type="text/javascript">
    //<![CDATA[
        function delete_session(id, text) {
            if ( confirm("<?php print_string('reallydelete', 'stupla'); ?>".replace(/\{\$a\}/, text)) )
                window.open('prot_userlist.php?stupla=<?php echo $stupla->id ?>&deletesession='+id, '_self');
            return true;
        }
    
        function undelete_session(id) {
            window.open('prot_userlist.php?stupla=<?php echo $stupla->id ?>&undeletesession='+id, '_self');
            return true;
        }
    //]]>
    </script><?php
    if ( $undeletemode == 1 )
        echo '<h1>', print_string('undeletemode', 'stupla'), '</h1>';
    echo '<form method="post">';
    print_tables_html($tables);
    echo $sError;
    if ( $undeletemode == 0 ) {?>
        <input type="button" value="<?php print_string('undeletemode', 'stupla')?>" onclick="window.open('prot_userlist.php?stupla=<?php echo $stupla->id ?>&undeletemode=1', '_self'); return true;" />
    <?php }
    echo '</form>';
    echo $OUTPUT->footer();
}
else if ( $format == "downloadascsv" )
    print_tables_csv('UserList', $tables);
else if ( $format == "downloadasexcel" )
    print_tables_xls('UserList', $tables);
?>
