<?php

function get_anchor($table) {
    return isset($table->anchor) ? $table->anchor : $table->name;
}

// ordering of tables

function order_tables(&$tables, $orderInfo) {
    $info = explode(',', $orderInfo); // anchor, column, direction, type
    for ($i = 0; $i < count($tables); $i++)
        if ( get_anchor($tables[$i]) == $info[0] )
            order_table($tables[$i], $info[1], $info[2], $info[3]);
}

function order_table(&$table, $col, $dir, $type)
{
    $help = array();
    foreach ($table->data as $line) {
        $v = $line[$col];
        if ( $type == 'n' ) { // numeric
            $v = 0+$v;
        }
        else if ( $type == 'p' ) { // percent
            $v = 0+substr($v, 0, -1);
        }
        else if ( $type == 'a' ) { // text with links
            $v = strtolower(preg_replace("/(<\/?)(\w+)([^>]*>)/", "", $v));
        }
        else if ( $type == 'd' ) { // date
            $s = preg_replace("/(<\/?)(\w+)([^>]*>)/", "", $v);
            $s = preg_replace("/[\[\]]/", "", $s);
            $d = explode('.', $s);
            $v = count($d) > 2 ? mktime(0, 0, 0, 0+$d[1], 0+$d[0], 2000+$d[2]) : '';
        }
        else if ( $type == 't' ) { // time
            $d = explode(':', $v);
            $v = $d[0]*60+$d[1];
        }
        else if ( is_string($v) ) {
            $v =strtolower($v);
        }
        $help[] = $v;
    }
    array_multisort($help, $dir == "d" ? SORT_DESC : SORT_ASC, $table->data);
}

// order headers

function makeorderhead_tables(&$tables, $orderInfo) {
    $info = explode(',', $orderInfo.",,,,,"); // anchor, column, direction, type
//    foreach ($tables as &$table)
//        makeorderhead_table($table, get_anchor($table) == $info[0] ? $info[1] : -1, $info[2], $info[3]);
    for ($i = 0; $i < count($tables); $i++)
        makeorderhead_table($tables[$i], get_anchor($tables[$i]) == $info[0] ? $info[1] : -1, $info[2], $info[3]);
}

function makeorderhead_table(&$table, $col, $dir, $type)
{
    $anchor = get_anchor($table);
    for ($i = 0; $i < count($table->head); $i++) {
        $add = "";
        if ( $col == $i && $dir == 'u' ) {
            $add .= '<a href="javascript:top.control.TableSort(\''.$anchor.','.$i.',u,'.
                   (isset($table->ordertype) ? $table->ordertype[$i] : '').'\')" target="control" style="color:red">&uarr;</a>';
        }
        else {
            $add .= '<a href="javascript:top.control.TableSort(\''.$anchor.','.$i.',u,'.
                   (isset($table->ordertype) ? $table->ordertype[$i] : '').'\')" target="control">&uarr;</a>';
        }
        if ( $col == $i && $dir == 'd' ) {
            $add .= '<a href="javascript:top.control.TableSort(\''.$anchor.','.$i.',d,'.
                   (isset($table->ordertype) ? $table->ordertype[$i] : '').'\')" target="control" style="color:red">&darr;</a>';
        }
        else {
            $add .= '<a href="javascript:top.control.TableSort(\''.$anchor.','.$i.',d,'.
                   (isset($table->ordertype) ? $table->ordertype[$i] : '').'\')" target="control">&darr;</a>';
        }
        $table->head[$i] .= $add;
    }
}

// print tables

function print_tables_html($tables) {
    global $sortinfo;
    if ( $sortinfo != '' )
        order_tables($tables, $sortinfo);
    makeorderhead_tables($tables, $sortinfo);

    foreach ($tables as $table) {
        ?><p style="text-align:center"><a name="<?php echo get_anchor($table) ?>"></a><b><?php echo $table->name; ?></b></p><?php
        echo html_writer::table($table);
    }
}

define('S2_EXCELROWS', 65535);
define('S2_FIRSTUSEDEXCELROW', 3);

function print_tables_xls($filenamePrefix, $tables) {
    global $CFG;
    global $sortinfo;
    if ( $sortinfo != '' )
        order_tables($tables, $sortinfo);

    // create Excel workbook
    if (file_exists("$CFG->libdir/excellib.class.php")) {
        // Moodle >= 1.6
        require_once("$CFG->libdir/excellib.class.php");
        $workbook = new MoodleExcelWorkbook("-");
        $wsnamelimit = 0; // no limit
    } else {
        // Moodle <= 1.5
        require_once("$CFG->libdir/excel/Worksheet.php");
        require_once("$CFG->libdir/excel/Workbook.php");
        $workbook = new Workbook("-");
        $wsnamelimit = 31; // max length in chars
    }

    $strftimedatetime = get_string("strftimedatetime");

    $nroPages = 0;
    foreach ($tables as $table) {
        $nroPages += ceil(count($table->data)/(S2_EXCELROWS-S2_FIRSTUSEDEXCELROW+1));
    }
    $filename = $filenamePrefix.'_'.userdate(time(),'%Y%m%d-%H%M',99,false);
    $filename .= '.xls';

    if (method_exists($workbook, 'send')) {
        // Moodle >=1.6
        $workbook->send($filename);
    } else {
        // Moodle <=1.5
        header("Content-type: application/vnd.ms-excel");
        header("Content-Disposition: attachment; filename=$filename" );
        header("Expires: 0");
        header("Cache-Control: must-revalidate, post-check=0,pre-check=0");
        header("Pragma: public");
    }


    $worksheet = array();

    $formatBold =& $workbook->add_format(array('bold'=>1));
    //$formatDate->set_num_format(get_string('log_excel_date_format'));

    // Creating worksheets
    $wsnumber = 1;
    foreach ($tables as $table) {
        $nPageTable = ceil(count($table->data)/(S2_EXCELROWS-S2_FIRSTUSEDEXCELROW+1));
        for ($j = 1; $j <= $nPageTable; $j++) {
        $sheettitle = isset($table->shortname) ? $table->shortname : $table->name;
        if ( $nPageTable > 1 )
            $sheettitle .= "_".$j;
        $worksheet[$wsnumber] =& $workbook->add_worksheet($sheettitle);
        $worksheet[$wsnumber]->set_column(1, 1, 30);
        $worksheet[$wsnumber]->write_string(0, 0, get_string('savedat').
                                    userdate(time(), $strftimedatetime));
        $worksheet[$wsnumber]->write_string(1, 0, $table->name);
        $col = 0;
        foreach ($table->head as $item) {
            $worksheet[$wsnumber]->write(S2_FIRSTUSEDEXCELROW-1,$col,$item,$formatBold);
            $col++;
        }
            $wsnumber++;
        }
    }

    $row = S2_FIRSTUSEDEXCELROW;
    $wsnumber = 1;
    $myxls =& $worksheet[$wsnumber];
    foreach ($tables as $table) {
        foreach ($table->data as $line) {
        if ($row > S2_EXCELROWS) {
          $wsnumber++;
          $myxls =& $worksheet[$wsnumber];
          $row = S2_FIRSTUSEDEXCELROW;
        }
        for ($i = 0; $i < count($line); $i++) {
            $myxls->write($row, $i, $line[$i], '');
        }
        $row++;
        }
      $wsnumber++;
      $myxls =& $worksheet[$wsnumber];
      $row = S2_FIRSTUSEDEXCELROW;
    }

    $workbook->close();
    exit;
}

function print_tables_csv($filenamePrefix, $tables) {
    global $sortinfo;
    if ( $sortinfo != '' )
        order_tables($tables, $sortinfo);

    $filename = $filenamePrefix.'_'.userdate(time(),'%Y%m%d-%H%M',99,false);
    $filename .= '.txt';
    header("Content-Type: application/download\n");
    header("Content-Disposition: attachment; filename=$filename");
    header("Expires: 0");
    header("Cache-Control: must-revalidate,post-check=0,pre-check=0");
    header("Pragma: public");

    $strftimedatetime = get_string("strftimedatetime");
    echo get_string('savedat').userdate(time(), $strftimedatetime)."\r\n";

        $first = true;
    foreach ($tables as $table) {
        if ( $first )
            $first = false;
        else
          echo "==================================================================================\r\n";
      echo implode("\t", $table->head), "\r\n";
        foreach ($table->data as $line) {
          echo implode("\t", array_map("safe_csv_text", $line)), "\r\n";
        }
    }
    exit;
}

function safe_csv_text($s)
{
    $hasNLs = (strpos($s, "\n") !== FALSE || strpos($s, "\r") !== FALSE);
    $hasQuotes = (strpos($s, '"') !== FALSE);
    if ( $hasNLs || $hasQuotes ) {
        if ( $hasQuotes )
            $s = str_replace('"', '\\"', $s);
        if ( $hasNLs )
            $s = preg_replace("/(\r|)\n/", '////', $s);
        return '"'.$s.'"';
    }
    return $s;
}

?>
